/* Generated file (by generator) */

/*
 * SPDX-FileCopyrightText: 2015 William Yu <williamyu@gnome.org>
 *
 * SPDX-License-Identifier: LGPL-2.1-only OR MPL-2.0
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "i-cal-vcard-restriction.h"
#include "libical-glib-private.h"
#include "i-cal-vcard-component.h"

/**
 * i_cal_vcard_restriction_compare:
 * @restr: The restriction kind
 * @count: The number of restrictions
 *
 * Checks if the given @count is in accordance with the given restriction, @restr.
 *
 * Returns: 1 if the restriction is met, 0 if not.
 *
 * Since: 1.0
 *
 **/
gint
i_cal_vcard_restriction_compare (ICalVcardRestrictionKind restr, 
                                 gint count)
{
	return vcardrestriction_compare ((vcardrestriction_kind) (restr), count);
}

/**
 * i_cal_vcard_restriction_check:
 * @comp: The component to be checked
 *
 * Checks whether the @comp is valid.
 *
 * Returns: Whether the @comp is valid.
 *
 * Since: 1.0
 *
 **/
gint
i_cal_vcard_restriction_check (ICalVcardComponent *comp)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_COMPONENT ((ICalVcardComponent*)comp), 0);

	return vcardrestriction_check ((vcardcomponent *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)comp)));
}
