/* Generated file (by generator) */

/*
 * SPDX-FileCopyrightText: 2015 William Yu <williamyu@gnome.org>
 *
 * SPDX-License-Identifier: LGPL-2.1-only OR MPL-2.0
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "i-cal-vcard-derived-property.h"
#include "libical-glib-private.h"
#include "i-cal-vcard-derived-value.h"
#include "i-cal-vcard-time.h"
#include "i-cal-vcard-str-array.h"
#include "i-cal-vcard-structured.h"
#include "i-cal-vcard-tz.h"
#include "i-cal-vcard-property.h"
#include "i-cal-vcard-geo.h"

/**
 * i_cal_vcard_property_new_adr:
 * @value: the #ICalVcardStructured value
 *
 * Creates a new #ICalVcardProperty holding ADR.
 *
 * Note: this uses a copy of the structured type, not
 * the one stored in the @value
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding ADR
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_adr (const ICalVcardStructured *value)
{
    vcardstructuredtype *val;
    g_return_val_if_fail(value != NULL, NULL);
    val = (vcardstructuredtype *)i_cal_object_get_native((ICalObject *)value);
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_property_new_full(vcardproperty_new_adr(vcardstructured_clone(val)), NULL);
}

/**
 * i_cal_vcard_property_get_adr:
 * @self: the #ICalVcardProperty of ADR
 *
 * Returns value of the ADR @self
 *
 * Note: this uses a copy of the structured type, not the one stored
 * in the @self
 *
 * Returns: (transfer full): the ADR value
 *
 * Since: 4.0
 *
 **/
ICalVcardStructured *
i_cal_vcard_property_get_adr (const ICalVcardProperty *self)
{
    vcardstructuredtype *val;
    g_return_val_if_fail(self != NULL, NULL);
    val = vcardproperty_get_adr((vcardproperty *)i_cal_object_get_native((ICalObject *)self));
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_structured_new_full(vcardstructured_clone(val), NULL);
}

/**
 * i_cal_vcard_property_set_adr:
 * @self: the #ICalVcardProperty of ADR
 * @value: the ADR value
 *
 * Sets value to the ADR @self
 *
 * Note: this uses a copy of the structured type, not the one stored in
 * the @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_adr (ICalVcardProperty *self, 
                              const ICalVcardStructured *value)
{
    g_return_if_fail(self != NULL);
    vcardproperty_set_adr((vcardproperty *)i_cal_object_get_native((ICalObject *)self),
        vcardstructured_clone((vcardstructuredtype *)i_cal_object_get_native((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_agent:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding AGENT.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding AGENT
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_agent (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_agent (value), NULL);
}

/**
 * i_cal_vcard_property_get_agent:
 * @self: the #ICalVcardProperty of AGENT
 *
 * Returns value of the AGENT @self
 *
 * Returns: the AGENT value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_agent (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_agent ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_agent:
 * @self: the #ICalVcardProperty of AGENT
 * @value: the AGENT value
 *
 * Sets value to the AGENT @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_agent (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_agent ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_anniversary:
 * @value: the #ICalVcardTime value
 *
 * Creates a new #ICalVcardProperty holding ANNIVERSARY.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding ANNIVERSARY
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_anniversary (const ICalVcardTime *value)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value), NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_anniversary (* (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value))), NULL);
}

/**
 * i_cal_vcard_property_get_anniversary:
 * @self: the #ICalVcardProperty of ANNIVERSARY
 *
 * Returns value of the ANNIVERSARY @self
 *
 * Returns: (transfer full): the ANNIVERSARY value
 *
 * Since: 4.0
 *
 **/
ICalVcardTime *
i_cal_vcard_property_get_anniversary (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return i_cal_vcard_time_new_full (vcardproperty_get_anniversary ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_anniversary:
 * @self: the #ICalVcardProperty of ANNIVERSARY
 * @value: the ANNIVERSARY value
 *
 * Sets value to the ANNIVERSARY @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_anniversary (ICalVcardProperty *self, 
                                      const ICalVcardTime *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value));

	vcardproperty_set_anniversary ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), * (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_bday:
 * @value: the #ICalVcardTime value
 *
 * Creates a new #ICalVcardProperty holding BDAY.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding BDAY
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_bday (const ICalVcardTime *value)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value), NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_bday (* (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value))), NULL);
}

/**
 * i_cal_vcard_property_get_bday:
 * @self: the #ICalVcardProperty of BDAY
 *
 * Returns value of the BDAY @self
 *
 * Returns: (transfer full): the BDAY value
 *
 * Since: 4.0
 *
 **/
ICalVcardTime *
i_cal_vcard_property_get_bday (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return i_cal_vcard_time_new_full (vcardproperty_get_bday ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_bday:
 * @self: the #ICalVcardProperty of BDAY
 * @value: the BDAY value
 *
 * Sets value to the BDAY @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_bday (ICalVcardProperty *self, 
                               const ICalVcardTime *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value));

	vcardproperty_set_bday ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), * (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_birthplace:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding BIRTHPLACE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding BIRTHPLACE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_birthplace (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_birthplace (value), NULL);
}

/**
 * i_cal_vcard_property_get_birthplace:
 * @self: the #ICalVcardProperty of BIRTHPLACE
 *
 * Returns value of the BIRTHPLACE @self
 *
 * Returns: the BIRTHPLACE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_birthplace (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_birthplace ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_birthplace:
 * @self: the #ICalVcardProperty of BIRTHPLACE
 * @value: the BIRTHPLACE value
 *
 * Sets value to the BIRTHPLACE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_birthplace (ICalVcardProperty *self, 
                                     const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_birthplace ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_caladruri:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding CALADRURI.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding CALADRURI
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_caladruri (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_caladruri (value), NULL);
}

/**
 * i_cal_vcard_property_get_caladruri:
 * @self: the #ICalVcardProperty of CALADRURI
 *
 * Returns value of the CALADRURI @self
 *
 * Returns: the CALADRURI value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_caladruri (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_caladruri ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_caladruri:
 * @self: the #ICalVcardProperty of CALADRURI
 * @value: the CALADRURI value
 *
 * Sets value to the CALADRURI @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_caladruri (ICalVcardProperty *self, 
                                    const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_caladruri ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_caluri:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding CALURI.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding CALURI
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_caluri (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_caluri (value), NULL);
}

/**
 * i_cal_vcard_property_get_caluri:
 * @self: the #ICalVcardProperty of CALURI
 *
 * Returns value of the CALURI @self
 *
 * Returns: the CALURI value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_caluri (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_caluri ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_caluri:
 * @self: the #ICalVcardProperty of CALURI
 * @value: the CALURI value
 *
 * Sets value to the CALURI @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_caluri (ICalVcardProperty *self, 
                                 const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_caluri ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_categories:
 * @value: (transfer none): the string array as a value for the new ICalVcardProperty holding CATEGORIES
 *
 * Creates a new #ICalVcardProperty holding CATEGORIES.
 *
 * Note: this uses a copy of the string array,
 * not the one stored in the @value
 *
 * Returns: (transfer full) (nullable): a new #ICalVcardProperty holding CATEGORIES
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_categories (const ICalVcardStrArray *value)
{
    vcardstrarray *val;
    g_return_val_if_fail(value != NULL, NULL);
    val = (vcardstrarray *)i_cal_object_get_native((ICalObject *)value);
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_property_new_full(vcardproperty_new_categories(vcardstrarray_clone(val)), NULL);
}

/**
 * i_cal_vcard_property_get_categories:
 * @self: the #ICalVcardProperty of CATEGORIES
 *
 * Gets the value of the @self of CATEGORIES.
 *
 * Note: this returns copy of the string array, not the
 * one stored in the @self
 *
 * Returns: (transfer full): the #ICalVcardStrArray value of the @self
 *
 * Since: 4.0
 *
 **/
ICalVcardStrArray *
i_cal_vcard_property_get_categories (const ICalVcardProperty *self)
{
    vcardstrarray *val;
    g_return_val_if_fail(self != NULL, NULL);
    val = vcardproperty_get_categories((vcardproperty *)i_cal_object_get_native((ICalObject *)self));
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_str_array_new_full(vcardstrarray_clone(val), NULL);
}

/**
 * i_cal_vcard_property_set_categories:
 * @self: the #ICalVcardProperty of CATEGORIES
 * @value: the string array value
 *
 * Sets the value of the @self of CATEGORIES.
 *
 * Note: this uses a copy of the string array, not the
 * one stored in the @value
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_categories (ICalVcardProperty *self, 
                                     const ICalVcardStrArray *value)
{
    g_return_if_fail(self != NULL);
    vcardproperty_set_categories((vcardproperty *)i_cal_object_get_native((ICalObject *)self),
        vcardstrarray_clone((vcardstrarray *)i_cal_object_get_native((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_class:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding CLASS.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding CLASS
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_class (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_class (value), NULL);
}

/**
 * i_cal_vcard_property_get_class:
 * @self: the #ICalVcardProperty of CLASS
 *
 * Returns value of the CLASS @self
 *
 * Returns: the CLASS value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_class (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_class ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_class:
 * @self: the #ICalVcardProperty of CLASS
 * @value: the CLASS value
 *
 * Sets value to the CLASS @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_class (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_class ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_clientpidmap:
 * @value: (transfer none): the string array as a value for the new ICalVcardProperty holding CLIENTPIDMAP
 *
 * Creates a new #ICalVcardProperty holding CLIENTPIDMAP.
 *
 * Note: this uses a copy of the string array,
 * not the one stored in the @value
 *
 * Returns: (transfer full) (nullable): a new #ICalVcardProperty holding CLIENTPIDMAP
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_clientpidmap (const ICalVcardStrArray *value)
{
    vcardstrarray *val;
    g_return_val_if_fail(value != NULL, NULL);
    val = (vcardstrarray *)i_cal_object_get_native((ICalObject *)value);
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_property_new_full(vcardproperty_new_clientpidmap(vcardstrarray_clone(val)), NULL);
}

/**
 * i_cal_vcard_property_get_clientpidmap:
 * @self: the #ICalVcardProperty of CLIENTPIDMAP
 *
 * Gets the value of the @self of CLIENTPIDMAP.
 *
 * Note: this returns copy of the string array, not the
 * one stored in the @self
 *
 * Returns: (transfer full): the #ICalVcardStrArray value of the @self
 *
 * Since: 4.0
 *
 **/
ICalVcardStrArray *
i_cal_vcard_property_get_clientpidmap (const ICalVcardProperty *self)
{
    vcardstrarray *val;
    g_return_val_if_fail(self != NULL, NULL);
    val = vcardproperty_get_clientpidmap((vcardproperty *)i_cal_object_get_native((ICalObject *)self));
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_str_array_new_full(vcardstrarray_clone(val), NULL);
}

/**
 * i_cal_vcard_property_set_clientpidmap:
 * @self: the #ICalVcardProperty of CLIENTPIDMAP
 * @value: the string array value
 *
 * Sets the value of the @self of CLIENTPIDMAP.
 *
 * Note: this uses a copy of the string array, not the
 * one stored in the @value
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_clientpidmap (ICalVcardProperty *self, 
                                       const ICalVcardStrArray *value)
{
    g_return_if_fail(self != NULL);
    vcardproperty_set_clientpidmap((vcardproperty *)i_cal_object_get_native((ICalObject *)self),
        vcardstrarray_clone((vcardstrarray *)i_cal_object_get_native((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_contacturi:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding CONTACTURI.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding CONTACTURI
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_contacturi (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_contacturi (value), NULL);
}

/**
 * i_cal_vcard_property_get_contacturi:
 * @self: the #ICalVcardProperty of CONTACTURI
 *
 * Returns value of the CONTACTURI @self
 *
 * Returns: the CONTACTURI value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_contacturi (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_contacturi ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_contacturi:
 * @self: the #ICalVcardProperty of CONTACTURI
 * @value: the CONTACTURI value
 *
 * Sets value to the CONTACTURI @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_contacturi (ICalVcardProperty *self, 
                                     const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_contacturi ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_created:
 * @value: the #ICalVcardTime value
 *
 * Creates a new #ICalVcardProperty holding CREATED.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding CREATED
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_created (const ICalVcardTime *value)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value), NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_created (* (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value))), NULL);
}

/**
 * i_cal_vcard_property_get_created:
 * @self: the #ICalVcardProperty of CREATED
 *
 * Returns value of the CREATED @self
 *
 * Returns: (transfer full): the CREATED value
 *
 * Since: 4.0
 *
 **/
ICalVcardTime *
i_cal_vcard_property_get_created (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return i_cal_vcard_time_new_full (vcardproperty_get_created ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_created:
 * @self: the #ICalVcardProperty of CREATED
 * @value: the CREATED value
 *
 * Sets value to the CREATED @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_created (ICalVcardProperty *self, 
                                  const ICalVcardTime *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value));

	vcardproperty_set_created ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), * (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_deathdate:
 * @value: the #ICalVcardTime value
 *
 * Creates a new #ICalVcardProperty holding DEATHDATE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding DEATHDATE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_deathdate (const ICalVcardTime *value)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value), NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_deathdate (* (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value))), NULL);
}

/**
 * i_cal_vcard_property_get_deathdate:
 * @self: the #ICalVcardProperty of DEATHDATE
 *
 * Returns value of the DEATHDATE @self
 *
 * Returns: (transfer full): the DEATHDATE value
 *
 * Since: 4.0
 *
 **/
ICalVcardTime *
i_cal_vcard_property_get_deathdate (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return i_cal_vcard_time_new_full (vcardproperty_get_deathdate ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_deathdate:
 * @self: the #ICalVcardProperty of DEATHDATE
 * @value: the DEATHDATE value
 *
 * Sets value to the DEATHDATE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_deathdate (ICalVcardProperty *self, 
                                    const ICalVcardTime *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value));

	vcardproperty_set_deathdate ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), * (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_deathplace:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding DEATHPLACE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding DEATHPLACE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_deathplace (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_deathplace (value), NULL);
}

/**
 * i_cal_vcard_property_get_deathplace:
 * @self: the #ICalVcardProperty of DEATHPLACE
 *
 * Returns value of the DEATHPLACE @self
 *
 * Returns: the DEATHPLACE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_deathplace (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_deathplace ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_deathplace:
 * @self: the #ICalVcardProperty of DEATHPLACE
 * @value: the DEATHPLACE value
 *
 * Sets value to the DEATHPLACE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_deathplace (ICalVcardProperty *self, 
                                     const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_deathplace ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_email:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding EMAIL.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding EMAIL
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_email (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_email (value), NULL);
}

/**
 * i_cal_vcard_property_get_email:
 * @self: the #ICalVcardProperty of EMAIL
 *
 * Returns value of the EMAIL @self
 *
 * Returns: the EMAIL value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_email (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_email ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_email:
 * @self: the #ICalVcardProperty of EMAIL
 * @value: the EMAIL value
 *
 * Sets value to the EMAIL @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_email (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_email ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_expertise:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding EXPERTISE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding EXPERTISE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_expertise (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_expertise (value), NULL);
}

/**
 * i_cal_vcard_property_get_expertise:
 * @self: the #ICalVcardProperty of EXPERTISE
 *
 * Returns value of the EXPERTISE @self
 *
 * Returns: the EXPERTISE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_expertise (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_expertise ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_expertise:
 * @self: the #ICalVcardProperty of EXPERTISE
 * @value: the EXPERTISE value
 *
 * Sets value to the EXPERTISE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_expertise (ICalVcardProperty *self, 
                                    const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_expertise ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_fburl:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding FBURL.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding FBURL
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_fburl (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_fburl (value), NULL);
}

/**
 * i_cal_vcard_property_get_fburl:
 * @self: the #ICalVcardProperty of FBURL
 *
 * Returns value of the FBURL @self
 *
 * Returns: the FBURL value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_fburl (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_fburl ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_fburl:
 * @self: the #ICalVcardProperty of FBURL
 * @value: the FBURL value
 *
 * Sets value to the FBURL @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_fburl (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_fburl ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_fn:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding FN.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding FN
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_fn (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_fn (value), NULL);
}

/**
 * i_cal_vcard_property_get_fn:
 * @self: the #ICalVcardProperty of FN
 *
 * Returns value of the FN @self
 *
 * Returns: the FN value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_fn (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_fn ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_fn:
 * @self: the #ICalVcardProperty of FN
 * @value: the FN value
 *
 * Sets value to the FN @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_fn (ICalVcardProperty *self, 
                             const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_fn ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_gender:
 * @value: (transfer none): the string array as a value for the new ICalVcardProperty holding GENDER
 *
 * Creates a new #ICalVcardProperty holding GENDER.
 *
 * Note: this uses a copy of the string array, not
 * the one stored in the @value
 *
 * Returns: (transfer full) (nullable): a new #ICalVcardProperty holding GENDER
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_gender (const ICalVcardStrArray *value)
{
    vcardstrarray *val;
    g_return_val_if_fail(value != NULL, NULL);
    val = (vcardstrarray *)i_cal_object_get_native((ICalObject *)value);
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_property_new_full(vcardproperty_new_gender(vcardstrarray_clone(val)), NULL);
}

/**
 * i_cal_vcard_property_get_gender:
 * @self: the #ICalVcardProperty of GENDER
 *
 * Gets the value of the @self of GENDER.
 *
 * Note: this returns copy of the string array, not the one
 * stored in the @self
 *
 * Returns: (transfer full): the #ICalVcardStrArray value of the @self
 *
 * Since: 4.0
 *
 **/
ICalVcardStrArray *
i_cal_vcard_property_get_gender (const ICalVcardProperty *self)
{
    vcardstrarray *val;
    g_return_val_if_fail(self != NULL, NULL);
    val = vcardproperty_get_gender((vcardproperty *)i_cal_object_get_native((ICalObject *)self));
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_str_array_new_full(vcardstrarray_clone(val), NULL);
}

/**
 * i_cal_vcard_property_set_gender:
 * @self: the #ICalVcardProperty of GENDER
 * @value: the string array value
 *
 * Sets the value of the @self of GENDER.
 *
 * Note: this uses a copy of the string array, not the one
 * stored in the @value
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_gender (ICalVcardProperty *self, 
                                 const ICalVcardStrArray *value)
{
    g_return_if_fail(self != NULL);
    vcardproperty_set_gender((vcardproperty *)i_cal_object_get_native((ICalObject *)self),
        vcardstrarray_clone((vcardstrarray *)i_cal_object_get_native((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_geo:
 * @value: the #ICalVcardGeo value
 *
 * Creates a new #ICalVcardProperty holding GEO.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding GEO
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_geo (const ICalVcardGeo *value)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_GEO ((ICalVcardGeo*)value), NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_geo (* (vcardgeotype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value))), NULL);
}

/**
 * i_cal_vcard_property_get_geo:
 * @self: the #ICalVcardProperty of GEO
 *
 * Returns value of the GEO @self
 *
 * Returns: (transfer full): the GEO value
 *
 * Since: 4.0
 *
 **/
ICalVcardGeo *
i_cal_vcard_property_get_geo (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return i_cal_vcard_geo_new_full (vcardproperty_get_geo ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_geo:
 * @self: the #ICalVcardProperty of GEO
 * @value: the GEO value
 *
 * Sets value to the GEO @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_geo (ICalVcardProperty *self, 
                              const ICalVcardGeo *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (I_CAL_VCARD_IS_GEO ((ICalVcardGeo*)value));

	vcardproperty_set_geo ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), * (vcardgeotype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_gramgender:
 * @value: the #ICalVcardPropertyGramgender value
 *
 * Creates a new #ICalVcardProperty holding GRAMGENDER.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding GRAMGENDER
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_gramgender (ICalVcardPropertyGramgender value)
{
	return i_cal_vcard_property_new_full (vcardproperty_new_gramgender ((vcardproperty_gramgender) (value)), NULL);
}

/**
 * i_cal_vcard_property_get_gramgender:
 * @self: the #ICalVcardProperty of GRAMGENDER
 *
 * Returns value of the GRAMGENDER @self
 *
 * Returns: the GRAMGENDER value
 *
 * Since: 4.0
 *
 **/
ICalVcardPropertyGramgender
i_cal_vcard_property_get_gramgender (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), I_CAL_VCARD_GRAMGENDER_NONE);

	return (ICalVcardPropertyGramgender) (vcardproperty_get_gramgender ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_gramgender:
 * @self: the #ICalVcardProperty of GRAMGENDER
 * @value: the GRAMGENDER value
 *
 * Sets value to the GRAMGENDER @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_gramgender (ICalVcardProperty *self, 
                                     ICalVcardPropertyGramgender value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));

	vcardproperty_set_gramgender ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), (vcardproperty_gramgender) (value));
}

/**
 * i_cal_vcard_property_new_hobby:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding HOBBY.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding HOBBY
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_hobby (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_hobby (value), NULL);
}

/**
 * i_cal_vcard_property_get_hobby:
 * @self: the #ICalVcardProperty of HOBBY
 *
 * Returns value of the HOBBY @self
 *
 * Returns: the HOBBY value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_hobby (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_hobby ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_hobby:
 * @self: the #ICalVcardProperty of HOBBY
 * @value: the HOBBY value
 *
 * Sets value to the HOBBY @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_hobby (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_hobby ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_impp:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding IMPP.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding IMPP
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_impp (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_impp (value), NULL);
}

/**
 * i_cal_vcard_property_get_impp:
 * @self: the #ICalVcardProperty of IMPP
 *
 * Returns value of the IMPP @self
 *
 * Returns: the IMPP value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_impp (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_impp ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_impp:
 * @self: the #ICalVcardProperty of IMPP
 * @value: the IMPP value
 *
 * Sets value to the IMPP @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_impp (ICalVcardProperty *self, 
                               const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_impp ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_interest:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding INTEREST.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding INTEREST
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_interest (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_interest (value), NULL);
}

/**
 * i_cal_vcard_property_get_interest:
 * @self: the #ICalVcardProperty of INTEREST
 *
 * Returns value of the INTEREST @self
 *
 * Returns: the INTEREST value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_interest (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_interest ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_interest:
 * @self: the #ICalVcardProperty of INTEREST
 * @value: the INTEREST value
 *
 * Sets value to the INTEREST @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_interest (ICalVcardProperty *self, 
                                   const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_interest ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_jsprop:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding JSPROP.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding JSPROP
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_jsprop (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_jsprop (value), NULL);
}

/**
 * i_cal_vcard_property_get_jsprop:
 * @self: the #ICalVcardProperty of JSPROP
 *
 * Returns value of the JSPROP @self
 *
 * Returns: the JSPROP value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_jsprop (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_jsprop ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_jsprop:
 * @self: the #ICalVcardProperty of JSPROP
 * @value: the JSPROP value
 *
 * Sets value to the JSPROP @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_jsprop (ICalVcardProperty *self, 
                                 const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_jsprop ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_key:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding KEY.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding KEY
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_key (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_key (value), NULL);
}

/**
 * i_cal_vcard_property_get_key:
 * @self: the #ICalVcardProperty of KEY
 *
 * Returns value of the KEY @self
 *
 * Returns: the KEY value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_key (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_key ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_key:
 * @self: the #ICalVcardProperty of KEY
 * @value: the KEY value
 *
 * Sets value to the KEY @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_key (ICalVcardProperty *self, 
                              const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_key ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_kind:
 * @value: the #ICalVcardPropertyKindenum value
 *
 * Creates a new #ICalVcardProperty holding KIND.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding KIND
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_kind (ICalVcardPropertyKindenum value)
{
	return i_cal_vcard_property_new_full (vcardproperty_new_kind ((vcardproperty_kindenum) (value)), NULL);
}

/**
 * i_cal_vcard_property_get_kind:
 * @self: the #ICalVcardProperty of KIND
 *
 * Returns value of the KIND @self
 *
 * Returns: the KIND value
 *
 * Since: 4.0
 *
 **/
ICalVcardPropertyKindenum
i_cal_vcard_property_get_kind (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), I_CAL_VCARD_KIND_NONE);

	return (ICalVcardPropertyKindenum) (vcardproperty_get_kind ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_kind:
 * @self: the #ICalVcardProperty of KIND
 * @value: the KIND value
 *
 * Sets value to the KIND @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_kind (ICalVcardProperty *self, 
                               ICalVcardPropertyKindenum value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));

	vcardproperty_set_kind ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), (vcardproperty_kindenum) (value));
}

/**
 * i_cal_vcard_property_new_label:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding LABEL.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding LABEL
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_label (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_label (value), NULL);
}

/**
 * i_cal_vcard_property_get_label:
 * @self: the #ICalVcardProperty of LABEL
 *
 * Returns value of the LABEL @self
 *
 * Returns: the LABEL value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_label (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_label ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_label:
 * @self: the #ICalVcardProperty of LABEL
 * @value: the LABEL value
 *
 * Sets value to the LABEL @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_label (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_label ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_lang:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding LANG.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding LANG
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_lang (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_lang (value), NULL);
}

/**
 * i_cal_vcard_property_get_lang:
 * @self: the #ICalVcardProperty of LANG
 *
 * Returns value of the LANG @self
 *
 * Returns: the LANG value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_lang (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_lang ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_lang:
 * @self: the #ICalVcardProperty of LANG
 * @value: the LANG value
 *
 * Sets value to the LANG @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_lang (ICalVcardProperty *self, 
                               const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_lang ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_language:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding LANGUAGE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding LANGUAGE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_language (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_language (value), NULL);
}

/**
 * i_cal_vcard_property_get_language:
 * @self: the #ICalVcardProperty of LANGUAGE
 *
 * Returns value of the LANGUAGE @self
 *
 * Returns: the LANGUAGE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_language (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_language ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_language:
 * @self: the #ICalVcardProperty of LANGUAGE
 * @value: the LANGUAGE value
 *
 * Sets value to the LANGUAGE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_language (ICalVcardProperty *self, 
                                   const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_language ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_logo:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding LOGO.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding LOGO
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_logo (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_logo (value), NULL);
}

/**
 * i_cal_vcard_property_get_logo:
 * @self: the #ICalVcardProperty of LOGO
 *
 * Returns value of the LOGO @self
 *
 * Returns: the LOGO value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_logo (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_logo ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_logo:
 * @self: the #ICalVcardProperty of LOGO
 * @value: the LOGO value
 *
 * Sets value to the LOGO @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_logo (ICalVcardProperty *self, 
                               const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_logo ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_mailer:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding MAILER.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding MAILER
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_mailer (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_mailer (value), NULL);
}

/**
 * i_cal_vcard_property_get_mailer:
 * @self: the #ICalVcardProperty of MAILER
 *
 * Returns value of the MAILER @self
 *
 * Returns: the MAILER value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_mailer (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_mailer ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_mailer:
 * @self: the #ICalVcardProperty of MAILER
 * @value: the MAILER value
 *
 * Sets value to the MAILER @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_mailer (ICalVcardProperty *self, 
                                 const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_mailer ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_member:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding MEMBER.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding MEMBER
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_member (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_member (value), NULL);
}

/**
 * i_cal_vcard_property_get_member:
 * @self: the #ICalVcardProperty of MEMBER
 *
 * Returns value of the MEMBER @self
 *
 * Returns: the MEMBER value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_member (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_member ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_member:
 * @self: the #ICalVcardProperty of MEMBER
 * @value: the MEMBER value
 *
 * Sets value to the MEMBER @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_member (ICalVcardProperty *self, 
                                 const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_member ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_n:
 * @value: the #ICalVcardStructured value
 *
 * Creates a new #ICalVcardProperty holding N.
 *
 * Note: this uses a copy of the structured type, not
 * the one stored in the @value
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding N
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_n (const ICalVcardStructured *value)
{
    vcardstructuredtype *val;
    g_return_val_if_fail(value != NULL, NULL);
    val = (vcardstructuredtype *)i_cal_object_get_native((ICalObject *)value);
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_property_new_full(vcardproperty_new_n(vcardstructured_clone(val)), NULL);
}

/**
 * i_cal_vcard_property_get_n:
 * @self: the #ICalVcardProperty of N
 *
 * Returns value of the N @self
 *
 * Note: this uses a copy of the structured type, not the one stored
 * in the @self
 *
 * Returns: (transfer full): the N value
 *
 * Since: 4.0
 *
 **/
ICalVcardStructured *
i_cal_vcard_property_get_n (const ICalVcardProperty *self)
{
    vcardstructuredtype *val;
    g_return_val_if_fail(self != NULL, NULL);
    val = vcardproperty_get_n((vcardproperty *)i_cal_object_get_native((ICalObject *)self));
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_structured_new_full(vcardstructured_clone(val), NULL);
}

/**
 * i_cal_vcard_property_set_n:
 * @self: the #ICalVcardProperty of N
 * @value: the N value
 *
 * Sets value to the N @self
 *
 * Note: this uses a copy of the structured type, not the one stored in
 * the @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_n (ICalVcardProperty *self, 
                            const ICalVcardStructured *value)
{
    g_return_if_fail(self != NULL);
    vcardproperty_set_n((vcardproperty *)i_cal_object_get_native((ICalObject *)self),
        vcardstructured_clone((vcardstructuredtype *)i_cal_object_get_native((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_name:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding NAME.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding NAME
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_name (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_name (value), NULL);
}

/**
 * i_cal_vcard_property_get_name:
 * @self: the #ICalVcardProperty of NAME
 *
 * Returns value of the NAME @self
 *
 * Returns: the NAME value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_name (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_name ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_name:
 * @self: the #ICalVcardProperty of NAME
 * @value: the NAME value
 *
 * Sets value to the NAME @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_name (ICalVcardProperty *self, 
                               const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_name ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_nickname:
 * @value: (transfer none): the string array as a value for the new ICalVcardProperty holding NICKNAME
 *
 * Creates a new #ICalVcardProperty holding NICKNAME.
 *
 * Note: this uses a copy of the string array,
 * not the one stored in the @value
 *
 * Returns: (transfer full) (nullable): a new #ICalVcardProperty holding NICKNAME
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_nickname (const ICalVcardStrArray *value)
{
    vcardstrarray *val;
    g_return_val_if_fail(value != NULL, NULL);
    val = (vcardstrarray *)i_cal_object_get_native((ICalObject *)value);
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_property_new_full(vcardproperty_new_nickname(vcardstrarray_clone(val)), NULL);
}

/**
 * i_cal_vcard_property_get_nickname:
 * @self: the #ICalVcardProperty of NICKNAME
 *
 * Gets the value of the @self of NICKNAME.
 *
 * Note: this returns copy of the string array, not the one
 * stored in the @self
 *
 * Returns: (transfer full): the #ICalVcardStrArray value of the @self
 *
 * Since: 4.0
 *
 **/
ICalVcardStrArray *
i_cal_vcard_property_get_nickname (const ICalVcardProperty *self)
{
    vcardstrarray *val;
    g_return_val_if_fail(self != NULL, NULL);
    val = vcardproperty_get_nickname((vcardproperty *)i_cal_object_get_native((ICalObject *)self));
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_str_array_new_full(vcardstrarray_clone(val), NULL);
}

/**
 * i_cal_vcard_property_set_nickname:
 * @self: the #ICalVcardProperty of NICKNAME
 * @value: the string array value
 *
 * Sets the value of the @self of NICKNAME.
 *
 * Note: this uses a copy of the string array, not the one
 * stored in the @value
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_nickname (ICalVcardProperty *self, 
                                   const ICalVcardStrArray *value)
{
    g_return_if_fail(self != NULL);
    vcardproperty_set_nickname((vcardproperty *)i_cal_object_get_native((ICalObject *)self),
        vcardstrarray_clone((vcardstrarray *)i_cal_object_get_native((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_note:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding NOTE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding NOTE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_note (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_note (value), NULL);
}

/**
 * i_cal_vcard_property_get_note:
 * @self: the #ICalVcardProperty of NOTE
 *
 * Returns value of the NOTE @self
 *
 * Returns: the NOTE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_note (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_note ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_note:
 * @self: the #ICalVcardProperty of NOTE
 * @value: the NOTE value
 *
 * Sets value to the NOTE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_note (ICalVcardProperty *self, 
                               const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_note ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_org:
 * @value: (transfer none): the string array as a value for the new ICalVcardProperty holding ORG
 *
 * Creates a new #ICalVcardProperty holding ORG.
 *
 * Note: this uses a copy of the string array, not the
 * one stored in the @value
 *
 * Returns: (transfer full) (nullable): a new #ICalVcardProperty holding ORG
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_org (const ICalVcardStrArray *value)
{
    vcardstrarray *val;
    g_return_val_if_fail(value != NULL, NULL);
    val = (vcardstrarray *)i_cal_object_get_native((ICalObject *)value);
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_property_new_full(vcardproperty_new_org(vcardstrarray_clone(val)), NULL);
}

/**
 * i_cal_vcard_property_get_org:
 * @self: the #ICalVcardProperty of ORG
 *
 * Gets the value of the @self of ORG.
 *
 * Note: this returns copy of the string array, not the one stored
 * in the @self
 *
 * Returns: (transfer full): the #ICalVcardStrArray value of the @self
 *
 * Since: 4.0
 *
 **/
ICalVcardStrArray *
i_cal_vcard_property_get_org (const ICalVcardProperty *self)
{
    vcardstrarray *val;
    g_return_val_if_fail(self != NULL, NULL);
    val = vcardproperty_get_org((vcardproperty *)i_cal_object_get_native((ICalObject *)self));
    if (val == NULL) {
        return NULL;
    }
    return i_cal_vcard_str_array_new_full(vcardstrarray_clone(val), NULL);
}

/**
 * i_cal_vcard_property_set_org:
 * @self: the #ICalVcardProperty of ORG
 * @value: the string array value
 *
 * Sets the value of the @self of ORG.
 *
 * Note: this uses a copy of the string array, not the one stored
 * in the @value
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_org (ICalVcardProperty *self, 
                              const ICalVcardStrArray *value)
{
    g_return_if_fail(self != NULL);
    vcardproperty_set_org((vcardproperty *)i_cal_object_get_native((ICalObject *)self),
        vcardstrarray_clone((vcardstrarray *)i_cal_object_get_native((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_orgdirectory:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding ORGDIRECTORY.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding ORGDIRECTORY
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_orgdirectory (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_orgdirectory (value), NULL);
}

/**
 * i_cal_vcard_property_get_orgdirectory:
 * @self: the #ICalVcardProperty of ORGDIRECTORY
 *
 * Returns value of the ORGDIRECTORY @self
 *
 * Returns: the ORGDIRECTORY value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_orgdirectory (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_orgdirectory ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_orgdirectory:
 * @self: the #ICalVcardProperty of ORGDIRECTORY
 * @value: the ORGDIRECTORY value
 *
 * Sets value to the ORGDIRECTORY @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_orgdirectory (ICalVcardProperty *self, 
                                       const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_orgdirectory ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_photo:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding PHOTO.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding PHOTO
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_photo (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_photo (value), NULL);
}

/**
 * i_cal_vcard_property_get_photo:
 * @self: the #ICalVcardProperty of PHOTO
 *
 * Returns value of the PHOTO @self
 *
 * Returns: the PHOTO value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_photo (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_photo ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_photo:
 * @self: the #ICalVcardProperty of PHOTO
 * @value: the PHOTO value
 *
 * Sets value to the PHOTO @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_photo (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_photo ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_prodid:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding PRODID.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding PRODID
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_prodid (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_prodid (value), NULL);
}

/**
 * i_cal_vcard_property_get_prodid:
 * @self: the #ICalVcardProperty of PRODID
 *
 * Returns value of the PRODID @self
 *
 * Returns: the PRODID value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_prodid (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_prodid ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_prodid:
 * @self: the #ICalVcardProperty of PRODID
 * @value: the PRODID value
 *
 * Sets value to the PRODID @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_prodid (ICalVcardProperty *self, 
                                 const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_prodid ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_pronouns:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding PRONOUNS.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding PRONOUNS
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_pronouns (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_pronouns (value), NULL);
}

/**
 * i_cal_vcard_property_get_pronouns:
 * @self: the #ICalVcardProperty of PRONOUNS
 *
 * Returns value of the PRONOUNS @self
 *
 * Returns: the PRONOUNS value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_pronouns (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_pronouns ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_pronouns:
 * @self: the #ICalVcardProperty of PRONOUNS
 * @value: the PRONOUNS value
 *
 * Sets value to the PRONOUNS @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_pronouns (ICalVcardProperty *self, 
                                   const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_pronouns ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_related:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding RELATED.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding RELATED
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_related (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_related (value), NULL);
}

/**
 * i_cal_vcard_property_get_related:
 * @self: the #ICalVcardProperty of RELATED
 *
 * Returns value of the RELATED @self
 *
 * Returns: the RELATED value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_related (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_related ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_related:
 * @self: the #ICalVcardProperty of RELATED
 * @value: the RELATED value
 *
 * Sets value to the RELATED @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_related (ICalVcardProperty *self, 
                                  const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_related ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_rev:
 * @value: the #ICalVcardTime value
 *
 * Creates a new #ICalVcardProperty holding REV.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding REV
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_rev (const ICalVcardTime *value)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value), NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_rev (* (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value))), NULL);
}

/**
 * i_cal_vcard_property_get_rev:
 * @self: the #ICalVcardProperty of REV
 *
 * Returns value of the REV @self
 *
 * Returns: (transfer full): the REV value
 *
 * Since: 4.0
 *
 **/
ICalVcardTime *
i_cal_vcard_property_get_rev (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return i_cal_vcard_time_new_full (vcardproperty_get_rev ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_rev:
 * @self: the #ICalVcardProperty of REV
 * @value: the REV value
 *
 * Sets value to the REV @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_rev (ICalVcardProperty *self, 
                              const ICalVcardTime *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (I_CAL_VCARD_IS_TIME ((ICalVcardTime*)value));

	vcardproperty_set_rev ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), * (vcardtimetype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_role:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding ROLE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding ROLE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_role (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_role (value), NULL);
}

/**
 * i_cal_vcard_property_get_role:
 * @self: the #ICalVcardProperty of ROLE
 *
 * Returns value of the ROLE @self
 *
 * Returns: the ROLE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_role (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_role ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_role:
 * @self: the #ICalVcardProperty of ROLE
 * @value: the ROLE value
 *
 * Sets value to the ROLE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_role (ICalVcardProperty *self, 
                               const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_role ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_socialprofile:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding SOCIALPROFILE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding SOCIALPROFILE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_socialprofile (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_socialprofile (value), NULL);
}

/**
 * i_cal_vcard_property_get_socialprofile:
 * @self: the #ICalVcardProperty of SOCIALPROFILE
 *
 * Returns value of the SOCIALPROFILE @self
 *
 * Returns: the SOCIALPROFILE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_socialprofile (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_socialprofile ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_socialprofile:
 * @self: the #ICalVcardProperty of SOCIALPROFILE
 * @value: the SOCIALPROFILE value
 *
 * Sets value to the SOCIALPROFILE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_socialprofile (ICalVcardProperty *self, 
                                        const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_socialprofile ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_sortstring:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding SORTSTRING.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding SORTSTRING
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_sortstring (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_sortstring (value), NULL);
}

/**
 * i_cal_vcard_property_get_sortstring:
 * @self: the #ICalVcardProperty of SORTSTRING
 *
 * Returns value of the SORTSTRING @self
 *
 * Returns: the SORTSTRING value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_sortstring (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_sortstring ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_sortstring:
 * @self: the #ICalVcardProperty of SORTSTRING
 * @value: the SORTSTRING value
 *
 * Sets value to the SORTSTRING @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_sortstring (ICalVcardProperty *self, 
                                     const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_sortstring ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_sound:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding SOUND.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding SOUND
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_sound (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_sound (value), NULL);
}

/**
 * i_cal_vcard_property_get_sound:
 * @self: the #ICalVcardProperty of SOUND
 *
 * Returns value of the SOUND @self
 *
 * Returns: the SOUND value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_sound (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_sound ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_sound:
 * @self: the #ICalVcardProperty of SOUND
 * @value: the SOUND value
 *
 * Sets value to the SOUND @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_sound (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_sound ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_source:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding SOURCE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding SOURCE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_source (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_source (value), NULL);
}

/**
 * i_cal_vcard_property_get_source:
 * @self: the #ICalVcardProperty of SOURCE
 *
 * Returns value of the SOURCE @self
 *
 * Returns: the SOURCE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_source (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_source ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_source:
 * @self: the #ICalVcardProperty of SOURCE
 * @value: the SOURCE value
 *
 * Sets value to the SOURCE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_source (ICalVcardProperty *self, 
                                 const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_source ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_tel:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding TEL.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding TEL
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_tel (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_tel (value), NULL);
}

/**
 * i_cal_vcard_property_get_tel:
 * @self: the #ICalVcardProperty of TEL
 *
 * Returns value of the TEL @self
 *
 * Returns: the TEL value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_tel (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_tel ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_tel:
 * @self: the #ICalVcardProperty of TEL
 * @value: the TEL value
 *
 * Sets value to the TEL @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_tel (ICalVcardProperty *self, 
                              const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_tel ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_title:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding TITLE.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding TITLE
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_title (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_title (value), NULL);
}

/**
 * i_cal_vcard_property_get_title:
 * @self: the #ICalVcardProperty of TITLE
 *
 * Returns value of the TITLE @self
 *
 * Returns: the TITLE value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_title (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_title ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_title:
 * @self: the #ICalVcardProperty of TITLE
 * @value: the TITLE value
 *
 * Sets value to the TITLE @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_title (ICalVcardProperty *self, 
                                const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_title ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_tz:
 * @value: the #ICalVcardTz value
 *
 * Creates a new #ICalVcardProperty holding TZ.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding TZ
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_tz (const ICalVcardTz *value)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_TZ ((ICalVcardTz*)value), NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_tz (* (vcardtztype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value))), NULL);
}

/**
 * i_cal_vcard_property_get_tz:
 * @self: the #ICalVcardProperty of TZ
 *
 * Returns value of the TZ @self
 *
 * Returns: (transfer full): the TZ value
 *
 * Since: 4.0
 *
 **/
ICalVcardTz *
i_cal_vcard_property_get_tz (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return i_cal_vcard_tz_new_full (vcardproperty_get_tz ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_tz:
 * @self: the #ICalVcardProperty of TZ
 * @value: the TZ value
 *
 * Sets value to the TZ @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_tz (ICalVcardProperty *self, 
                             const ICalVcardTz *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (I_CAL_VCARD_IS_TZ ((ICalVcardTz*)value));

	vcardproperty_set_tz ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), * (vcardtztype *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)value)));
}

/**
 * i_cal_vcard_property_new_uid:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding UID.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding UID
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_uid (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_uid (value), NULL);
}

/**
 * i_cal_vcard_property_get_uid:
 * @self: the #ICalVcardProperty of UID
 *
 * Returns value of the UID @self
 *
 * Returns: the UID value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_uid (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_uid ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_uid:
 * @self: the #ICalVcardProperty of UID
 * @value: the UID value
 *
 * Sets value to the UID @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_uid (ICalVcardProperty *self, 
                              const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_uid ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_url:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding URL.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding URL
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_url (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_url (value), NULL);
}

/**
 * i_cal_vcard_property_get_url:
 * @self: the #ICalVcardProperty of URL
 *
 * Returns value of the URL @self
 *
 * Returns: the URL value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_url (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_url ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_url:
 * @self: the #ICalVcardProperty of URL
 * @value: the URL value
 *
 * Sets value to the URL @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_url (ICalVcardProperty *self, 
                              const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_url ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_version:
 * @value: the #ICalVcardPropertyVersion value
 *
 * Creates a new #ICalVcardProperty holding VERSION.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding VERSION
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_version (ICalVcardPropertyVersion value)
{
	return i_cal_vcard_property_new_full (vcardproperty_new_version ((vcardproperty_version) (value)), NULL);
}

/**
 * i_cal_vcard_property_get_version:
 * @self: the #ICalVcardProperty of VERSION
 *
 * Returns value of the VERSION @self
 *
 * Returns: the VERSION value
 *
 * Since: 4.0
 *
 **/
ICalVcardPropertyVersion
i_cal_vcard_property_get_version (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), I_CAL_VCARD_VERSION_NONE);

	return (ICalVcardPropertyVersion) (vcardproperty_get_version ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self))));
}

/**
 * i_cal_vcard_property_set_version:
 * @self: the #ICalVcardProperty of VERSION
 * @value: the VERSION value
 *
 * Sets value to the VERSION @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_version (ICalVcardProperty *self, 
                                  ICalVcardPropertyVersion value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));

	vcardproperty_set_version ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), (vcardproperty_version) (value));
}

/**
 * i_cal_vcard_property_new_x:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding X.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding X
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_x (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_x (value), NULL);
}

/**
 * i_cal_vcard_property_get_x:
 * @self: the #ICalVcardProperty of X
 *
 * Returns value of the X @self
 *
 * Returns: the X value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_x (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_x ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_x:
 * @self: the #ICalVcardProperty of X
 * @value: the X value
 *
 * Sets value to the X @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_x (ICalVcardProperty *self, 
                            const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_x ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_xlicerror:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding XLICERROR.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding XLICERROR
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_xlicerror (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_xlicerror (value), NULL);
}

/**
 * i_cal_vcard_property_get_xlicerror:
 * @self: the #ICalVcardProperty of XLICERROR
 *
 * Returns value of the XLICERROR @self
 *
 * Returns: the XLICERROR value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_xlicerror (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_xlicerror ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_xlicerror:
 * @self: the #ICalVcardProperty of XLICERROR
 * @value: the XLICERROR value
 *
 * Sets value to the XLICERROR @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_xlicerror (ICalVcardProperty *self, 
                                    const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_xlicerror ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}

/**
 * i_cal_vcard_property_new_xml:
 * @value: the text value
 *
 * Creates a new #ICalVcardProperty holding XML.
 *
 * Returns: (transfer full): a new #ICalVcardProperty holding XML
 *
 * Since: 4.0
 *
 **/
ICalVcardProperty *
i_cal_vcard_property_new_xml (const gchar *value)
{
    g_return_val_if_fail (value != NULL, NULL);

	return i_cal_vcard_property_new_full (vcardproperty_new_xml (value), NULL);
}

/**
 * i_cal_vcard_property_get_xml:
 * @self: the #ICalVcardProperty of XML
 *
 * Returns value of the XML @self
 *
 * Returns: the XML value
 *
 * Since: 4.0
 *
 **/
const gchar *
i_cal_vcard_property_get_xml (const ICalVcardProperty *self)
{
    g_return_val_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self), NULL);

	return vcardproperty_get_xml ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)));
}

/**
 * i_cal_vcard_property_set_xml:
 * @self: the #ICalVcardProperty of XML
 * @value: the XML value
 *
 * Sets value to the XML @self
 *
 * Since: 4.0
 *
 **/
void
i_cal_vcard_property_set_xml (ICalVcardProperty *self, 
                              const gchar *value)
{
    g_return_if_fail (I_CAL_VCARD_IS_PROPERTY ((ICalVcardProperty*)self));
    g_return_if_fail (value != NULL);

	vcardproperty_set_xml ((vcardproperty *)i_cal_object_get_native (I_CAL_OBJECT ((ICalObject *)self)), value);
}
