/*
  SPDX-FileCopyrightText: 1999 Eric Busboom <eric@civicknowledge.com>
  SPDX-License-Identifier: LGPL-2.1-only OR MPL-2.0
*/

// NOLINTBEGIN

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wsign-conversion"
#pragma GCC diagnostic ignored "-Wimplicit-fallthrough"
#endif

#define YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer ss_create_buffer
#define yy_delete_buffer ss_delete_buffer
#define yy_flex_debug ss_flex_debug
#define yy_init_buffer ss_init_buffer
#define yy_flush_buffer ss_flush_buffer
#define yy_load_buffer_state ss_load_buffer_state
#define yy_switch_to_buffer ss_switch_to_buffer
#define yyin ssin
#define yyleng ssleng
#define yylex sslex
#define yylineno sslineno
#define yyout ssout
#define yyrestart ssrestart
#define yytext sstext
#define yywrap sswrap
#define yyalloc ssalloc
#define yyrealloc ssrealloc
#define yyfree ssfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <stddef.h> /* for ptrdiff_t */

#ifndef __cplusplus
extern int isatty(int);
#endif /* __cplusplus */

#ifdef _WIN32_WCE //krazy:exclude=cpp
#include <io.h>
#endif

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types.
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t;
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN (-32767 - 1)
#endif
#ifndef INT32_MIN
#define INT32_MIN (-2147483647 - 1)
#endif
#ifndef INT8_MAX
#define INT8_MAX (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else /* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined(__STDC__)

#define YY_USE_CONST

#endif /* defined (__STDC__) */
#endif /* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int)(unsigned char)c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE ssrestart(ssin)

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int ssleng;

extern FILE *ssin, *ssout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

#define YY_LESS_LINENO(n)

/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n)                                                                               \
    do {                                                                                        \
        /* Undo effects of setting up sstext. */                                                \
        int yyless_macro_arg = (n);                                                             \
        YY_LESS_LINENO(yyless_macro_arg);                                                       \
        *yy_cp = (yy_hold_char);                                                                \
        YY_RESTORE_YY_MORE_OFFSET(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
        YY_DO_BEFORE_ACTION; /* set up sstext again */                                          \
    } while (0)

#define unput(c) yyunput(c, (yytext_ptr))

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state {
    FILE *yy_input_file;

    char *yy_ch_buf;  /* input buffer */
    char *yy_buf_pos; /* current position in input buffer */

    /* Size of input buffer in bytes, not including room for EOB
         * characters.
         */
    yy_size_t yy_buf_size;

    /* Number of characters read into yy_ch_buf, not including EOB
         * characters.
         */
    int yy_n_chars;

    /* Whether we "own" the buffer - i.e., we know we created it,
         * and can realloc() it to grow it, and should free() it to
         * delete it.
         */
    int yy_is_our_buffer;

    /* Whether this is an "interactive" input source; if so, and
         * if we're using stdio for input, then we want to use getc()
         * instead of fread(), to make sure we stop fetching input after
         * each newline.
         */
    int yy_is_interactive;

    /* Whether we're considered to be at the beginning of a line.
         * If so, '^' rules will be active on the next match, otherwise
         * not.
         */
    int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

    /* Whether to try to fill the input buffer when we reach the
         * end of it.
         */
    int yy_fill_buffer;

    int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
    /* When an EOF's been seen but there's still some text to process
         * then we mark the buffer as YY_EOF_PENDING, to indicate that we
         * shouldn't try reading from the input source any more.  We might
         * still have a bunch of tokens to match, though, because of
         * possible backing-up.
         *
         * When we actually see the EOF, we change the status to "new"
         * (via ssrestart()), so that the user can continue scanning by
         * just pointing ssin at a new input file.
         */
#define YY_BUFFER_EOF_PENDING 2
};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0;       /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0;       /**< capacity of stack. */
static YY_BUFFER_STATE *yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ((yy_buffer_stack)                              \
                               ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                               : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when sstext is formed. */
static char yy_hold_char;
static int yy_n_chars; /* number of characters read into yy_ch_buf */
int ssleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *)0;
static int yy_init = 0;  /* whether we need to initialize */
static int yy_start = 0; /* start state number */

/* Flag which is used to allow sswrap()'s to do buffer switches
 * instead of setting up a fresh ssin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void ssrestart(FILE *input_file);
void ss_switch_to_buffer(YY_BUFFER_STATE new_buffer);
YY_BUFFER_STATE ss_create_buffer(FILE *file, int size);
void ss_delete_buffer(YY_BUFFER_STATE b);
void ss_flush_buffer(YY_BUFFER_STATE b);
void sspush_buffer_state(YY_BUFFER_STATE new_buffer);
void sspop_buffer_state(void);

static void ssensure_buffer_stack(void);
static void ss_load_buffer_state(void);
static void ss_init_buffer(YY_BUFFER_STATE b, FILE *file);

#define YY_FLUSH_BUFFER ss_flush_buffer(YY_CURRENT_BUFFER)

YY_BUFFER_STATE ss_scan_buffer(char *base, yy_size_t size);
YY_BUFFER_STATE ss_scan_string(yyconst char *yy_str);
YY_BUFFER_STATE ss_scan_bytes(yyconst char *bytes, int len);

void *ssalloc(yy_size_t);
void *ssrealloc(void *, yy_size_t);
void ssfree(void *);

#define yy_new_buffer ss_create_buffer

#define yy_set_interactive(is_interactive)                            \
    {                                                                 \
        if (!YY_CURRENT_BUFFER) {                                     \
            ssensure_buffer_stack();                                  \
            YY_CURRENT_BUFFER_LVALUE =                                \
                ss_create_buffer(ssin, YY_BUF_SIZE);                  \
        }                                                             \
        YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
    }

#define yy_set_bol(at_bol)                            \
    {                                                 \
        if (!YY_CURRENT_BUFFER) {                     \
            ssensure_buffer_stack();                  \
            YY_CURRENT_BUFFER_LVALUE =                \
                ss_create_buffer(ssin, YY_BUF_SIZE);  \
        }                                             \
        YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
    }

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *ssin = (FILE *)0, *ssout = (FILE *)0;

typedef int yy_state_type;

extern int sslineno;

int sslineno = 1;

extern char sstext[];

static yy_state_type yy_get_previous_state(void);
static yy_state_type yy_try_NUL_trans(yy_state_type current_state);
static int yy_get_next_buffer(void);
static void yy_fatal_error(yyconst char msg[]);

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up sstext.
 */
#define YY_DO_BEFORE_ACTION                                \
    (yytext_ptr) = yy_bp;                                  \
    ssleng = (int)(ptrdiff_t)(yy_cp - yy_bp);              \
    (yy_hold_char) = *yy_cp;                               \
    *yy_cp = '\0';                                         \
    if (ssleng >= YYLMAX)                                  \
        YY_FATAL_ERROR("token too large, exceeds YYLMAX"); \
    yy_flex_strncpy(sstext, (yytext_ptr), ssleng + 1);     \
    (yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 23
#define YY_END_OF_BUFFER 24
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info {
    flex_int32_t yy_verify;
    flex_int32_t yy_nxt;
};
static yyconst flex_int16_t yy_accept[56] =
    {0,
     0, 0, 0, 0, 0, 0, 24, 22, 18, 18,
     22, 17, 21, 4, 19, 8, 5, 9, 21, 21,
     21, 21, 21, 21, 21, 18, 7, 0, 21, 10,
     6, 11, 21, 21, 14, 21, 21, 13, 21, 21,
     20, 12, 21, 15, 21, 21, 21, 2, 16, 21,
     21, 21, 3, 1, 0};

static yyconst flex_int32_t yy_ec[256] =
    {0,
     1, 1, 1, 1, 1, 1, 1, 1, 2, 3,
     1, 1, 2, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 4, 5, 1, 1, 1, 1, 1, 6, 1,
     1, 7, 1, 8, 7, 7, 1, 7, 7, 7,
     7, 7, 7, 7, 7, 7, 7, 9, 10, 11,
     12, 13, 1, 7, 14, 7, 15, 16, 17, 18,
     7, 19, 20, 7, 7, 21, 22, 23, 24, 7,
     7, 25, 26, 27, 28, 7, 29, 7, 7, 7,
     1, 1, 1, 1, 1, 1, 30, 7, 31, 32,

     33, 34, 7, 35, 36, 7, 7, 37, 38, 39,
     40, 7, 7, 41, 42, 43, 44, 7, 45, 7,
     7, 7, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,

     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1};

static yyconst flex_int32_t yy_meta[46] =
    {0,
     1, 1, 1, 2, 1, 1, 3, 1, 2, 1,
     1, 1, 1, 3, 3, 3, 3, 3, 3, 3,
     3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
     3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
     3, 3, 3, 3, 3};

static yyconst flex_int16_t yy_base[58] =
    {0,
     0, 0, 0, 0, 0, 0, 98, 107, 44, 47,
     85, 0, 0, 107, 107, 84, 83, 79, 29, 28,
     28, 31, 31, 40, 39, 58, 107, 84, 0, 107,
     107, 107, 47, 40, 0, 38, 45, 0, 46, 59,
     107, 0, 55, 0, 57, 67, 60, 0, 0, 71,
     70, 61, 0, 0, 107, 103, 86};

static yyconst flex_int16_t yy_def[58] =
    {0,
     55, 1, 1, 1, 1, 1, 55, 55, 55, 55,
     55, 56, 57, 55, 55, 55, 55, 55, 57, 57,
     57, 57, 57, 57, 57, 55, 55, 56, 57, 55,
     55, 55, 57, 57, 57, 57, 57, 57, 57, 57,
     55, 57, 57, 57, 57, 57, 57, 57, 57, 57,
     57, 57, 57, 57, 0, 55, 55};

static yyconst flex_int16_t yy_nxt[153] =
    {0,
     8, 9, 10, 9, 11, 12, 13, 14, 8, 15,
     16, 17, 18, 19, 13, 13, 13, 20, 13, 21,
     13, 13, 22, 23, 13, 24, 13, 13, 25, 19,
     13, 13, 13, 20, 13, 21, 13, 13, 22, 23,
     13, 24, 13, 13, 25, 26, 26, 26, 26, 26,
     26, 33, 34, 35, 36, 38, 39, 40, 37, 26,
     26, 26, 42, 43, 44, 45, 46, 33, 34, 35,
     36, 38, 39, 40, 37, 47, 48, 49, 42, 43,
     44, 45, 46, 50, 51, 52, 53, 54, 29, 41,
     32, 47, 48, 49, 31, 30, 27, 55, 55, 50,

     51, 52, 53, 54, 28, 28, 7, 55, 55, 55,
     55, 55, 55, 55, 55, 55, 55, 55, 55, 55,
     55, 55, 55, 55, 55, 55, 55, 55, 55, 55,
     55, 55, 55, 55, 55, 55, 55, 55, 55, 55,
     55, 55, 55, 55, 55, 55, 55, 55, 55, 55,
     55, 55};

static yyconst flex_int16_t yy_chk[153] =
    {0,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 9, 9, 9, 10, 10,
     10, 19, 20, 21, 22, 23, 24, 25, 22, 26,
     26, 26, 33, 34, 36, 37, 39, 19, 20, 21,
     22, 23, 24, 25, 22, 40, 43, 45, 33, 34,
     36, 37, 39, 46, 47, 50, 51, 52, 57, 28,
     18, 40, 43, 45, 17, 16, 11, 7, 0, 46,

     47, 50, 51, 52, 56, 56, 55, 55, 55, 55,
     55, 55, 55, 55, 55, 55, 55, 55, 55, 55,
     55, 55, 55, 55, 55, 55, 55, 55, 55, 55,
     55, 55, 55, 55, 55, 55, 55, 55, 55, 55,
     55, 55, 55, 55, 55, 55, 55, 55, 55, 55,
     55, 55};

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int ss_flex_debug;
int ss_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#ifndef YYLMAX
#define YYLMAX 8192
#endif

char sstext[YYLMAX];
char *yytext_ptr;
/*
======================================================================
 FILE: icalsslexer.l
 CREATOR: eric 8 Aug 2000

 SPDX-FileCopyrightText: 2000, Eric Busboom <eric@civicknowledge.com>

 This program is free software; you can redistribute it and/or modify




 The Original Code is eric. The Initial Developer of the Original
 Code is Eric Busboom

======================================================================*/

#include "icalssyacc.h"
#include "icalmemory.h"

const char *input_buffer;
const char *input_buffer_p;

#define minInt(a, b) ((a) < (b) ? (a) : (b))

int icalss_input(char *buf, size_t max_size)
{
    size_t n = minInt(max_size, strlen(input_buffer_p));

    if (n > 0) {
        memcpy(buf, input_buffer_p, n);
        input_buffer_p += n;
        return (int)n;
    } else {
        return 0;
    }
}

#undef YY_INPUT
#define YY_INPUT(b, r, ms) (r = icalss_input(b, ms))

#undef SS_FATAL_ERROR
#define SS_FATAL_ERROR(msg) sserror(msg)

#define INITIAL 0
#define sql 1
#define string_value 2

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals(void);

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int sslex_destroy(void);

int ssget_debug(void);

void ssset_debug(int debug_flag);

YY_EXTRA_TYPE ssget_extra(void);

void ssset_extra(YY_EXTRA_TYPE user_defined);

FILE *ssget_in(void);

void ssset_in(FILE *in_str);

FILE *ssget_out(void);

void ssset_out(FILE *out_str);

int ssget_leng(void);

char *ssget_text(void);

int ssget_lineno(void);

void ssset_lineno(int line_number);

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int sswrap(void);
#else
extern int sswrap(void);
#endif
#endif

static void yyunput(int c, char *buf_ptr);

#ifndef yytext_ptr
static void yy_flex_strncpy(char *, yyconst char *, int);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen(yyconst char *);
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput(void);
#else
static int input(void);
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO                                    \
    do {                                        \
        if (fwrite(sstext, ssleng, 1, ssout)) { \
        }                                       \
    } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf, result, max_size)                                         \
    if (YY_CURRENT_BUFFER_LVALUE->yy_is_interactive) {                          \
        int c = '*';                                                            \
        size_t n;                                                               \
        for (n = 0; n < max_size &&                                             \
                    (c = getc(ssin)) != EOF && c != '\n';                       \
             ++n)                                                               \
            buf[n] = (char)c;                                                   \
        if (c == '\n')                                                          \
            buf[n++] = (char)c;                                                 \
        if (c == EOF && ferror(ssin))                                           \
            YY_FATAL_ERROR("input in flex scanner failed");                     \
        result = n;                                                             \
    } else {                                                                    \
        errno = 0;                                                              \
        while ((result = fread(buf, 1, max_size, ssin)) == 0 && ferror(ssin)) { \
            if (errno != EINTR) {                                               \
                YY_FATAL_ERROR("input in flex scanner failed");                 \
                break;                                                          \
            }                                                                   \
            errno = 0;                                                          \
            clearerr(ssin);                                                     \
        }                                                                       \
    }

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error(msg)
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int sslex(void);

#define YY_DECL int sslex(void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after sstext and ssleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
    YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
    register yy_state_type yy_current_state;
    register char *yy_cp, *yy_bp;
    register int yy_act;

    if (!(yy_init)) {
        (yy_init) = 1;

#ifdef YY_USER_INIT
        YY_USER_INIT;
#endif

        if (!(yy_start)) {
            (yy_start) = 1; /* first start state */
        }

        if (!ssin) {
            ssin = stdin;
        }

        if (!ssout) {
            ssout = stdout;
        }

        if (!YY_CURRENT_BUFFER) {
            ssensure_buffer_stack();
            YY_CURRENT_BUFFER_LVALUE =
                ss_create_buffer(ssin, YY_BUF_SIZE);
        }

        ss_load_buffer_state();
    }

    while (1) /* loops until end-of-file is reached */
    {
        yy_cp = (yy_c_buf_p);

        /* Support of sstext. */
        *yy_cp = (yy_hold_char);

        /* yy_bp points to the position in yy_ch_buf of the start of
                 * the current run.
                 */
        yy_bp = yy_cp;

        yy_current_state = (yy_start);
    yy_match:
        do {
            register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
            if (yy_accept[yy_current_state]) {
                (yy_last_accepting_state) = yy_current_state;
                (yy_last_accepting_cpos) = yy_cp;
            }
            while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
                yy_current_state = (int)yy_def[yy_current_state];
                if (yy_current_state >= 56) {
                    yy_c = yy_meta[(unsigned int)yy_c];
                }
            }
            yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int)yy_c];
            ++yy_cp;
        } while (yy_base[yy_current_state] != 107);

    yy_find_action:
        yy_act = yy_accept[yy_current_state];
        if (yy_act == 0) { /* have to back up */
            yy_cp = (yy_last_accepting_cpos);
            yy_current_state = (yy_last_accepting_state);
            yy_act = yy_accept[yy_current_state];
        }

        YY_DO_BEFORE_ACTION;

    do_action: /* This label is used only to access EOF actions. */

        switch (yy_act) { /* beginning of action switch */
        case 0:           /* must back up */
            /* undo the effects of YY_DO_BEFORE_ACTION */
            *yy_cp = (yy_hold_char);
            yy_cp = (yy_last_accepting_cpos);
            yy_current_state = (yy_last_accepting_state);
            goto yy_find_action;

        case 1:
            YY_RULE_SETUP
            {
                return SELECT;
            }
            YY_BREAK
        case 2:
            YY_RULE_SETUP
            {
                return FROM;
            }
            YY_BREAK
        case 3:
            YY_RULE_SETUP
            {
                return WHERE;
            }
            YY_BREAK
        case 4:
            YY_RULE_SETUP
            {
                return COMMA;
            }
            YY_BREAK
        case 5:
            YY_RULE_SETUP
            {
                return EQUALS;
            }
            YY_BREAK
        case 6:
            YY_RULE_SETUP
            {
                return EQUALS;
            }
            YY_BREAK
        case 7:
            YY_RULE_SETUP
            {
                return NOTEQUALS;
            }
            YY_BREAK
        case 8:
            YY_RULE_SETUP
            {
                return LESS;
            }
            YY_BREAK
        case 9:
            YY_RULE_SETUP
            {
                return GREATER;
            }
            YY_BREAK
        case 10:
            YY_RULE_SETUP
            {
                return LESSEQUALS;
            }
            YY_BREAK
        case 11:
            YY_RULE_SETUP
            {
                return GREATEREQUALS;
            }
            YY_BREAK
        case 12:
            YY_RULE_SETUP
            {
                return AND;
            }
            YY_BREAK
        case 13:
            YY_RULE_SETUP
            {
                return OR;
            }
            YY_BREAK
        case 14:
            YY_RULE_SETUP
            {
                return IS;
            }
            YY_BREAK
        case 15:
            YY_RULE_SETUP
            {
                return NOT;
            }
            YY_BREAK
        case 16:
            YY_RULE_SETUP
            {
                return SQLNULL;
            }
            YY_BREAK
        case 17:
            YY_RULE_SETUP
            {
                return QUOTE;
            }
            YY_BREAK
        case 18:
            /* rule 18 can match eol */
            YY_RULE_SETUP;
            YY_BREAK
        case 19:
            YY_RULE_SETUP
            {
                return EOL;
            }
            YY_BREAK
        case 20:
            YY_RULE_SETUP
            {
                int c = input();
                if (c != EOF) {
                    unput(c);
                }
                if (c != '\'') {
                    sslval.v_string = icalmemory_tmp_copy(sstext);
                    return STRING;
                } else {
                    /*ssmore();*/
                }
            }
            YY_BREAK
        case 21:
            YY_RULE_SETUP
            {
                sslval.v_string = icalmemory_tmp_copy(sstext);
                return STRING;
            }
            YY_BREAK
        case 22:
            YY_RULE_SETUP
            {
                return sstext[0];
            }
            YY_BREAK
        case 23:
            YY_RULE_SETUP
            ECHO;
            YY_BREAK
        case YY_STATE_EOF(INITIAL):
        case YY_STATE_EOF(sql):
        case YY_STATE_EOF(string_value):
            yyterminate();

        case YY_END_OF_BUFFER: {
            /* Amount of text matched not including the EOB char. */
            int yy_amount_of_matched_text = (int)(yy_cp - (yytext_ptr)) - 1;

            /* Undo the effects of YY_DO_BEFORE_ACTION. */
            *yy_cp = (yy_hold_char);
            YY_RESTORE_YY_MORE_OFFSET

            if (YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW) {
                /* We're scanning a new file or input source.  It's
                         * possible that this happened because the user
                         * just pointed ssin at a new source and called
                         * sslex().  If so, then we have to assure
                         * consistency between YY_CURRENT_BUFFER and our
                         * globals.  Here is the right place to do so, because
                         * this is the first action (other than possibly a
                         * back-up) that will match for the new input source.
                         */
                (yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
                YY_CURRENT_BUFFER_LVALUE->yy_input_file = ssin;
                YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
            }

            /* Note that here we test for yy_c_buf_p "<=" to the position
                 * of the first EOB in the buffer, since yy_c_buf_p will
                 * already have been incremented past the NUL character
                 * (since all states make transitions on EOB to the
                 * end-of-buffer state).  Contrast this with the test
                 * in input().
                 */
            if ((yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)]) { /* This was really a NUL. */
                yy_state_type yy_next_state;

                (yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

                yy_current_state = yy_get_previous_state();

                /* Okay, we're now positioned to make the NUL
                         * transition.  We couldn't have
                         * yy_get_previous_state() go ahead and do it
                         * for us because it doesn't know how to deal
                         * with the possibility of jamming (and we don't
                         * want to build jamming into it because then it
                         * will run more slowly).
                         */

                yy_next_state = yy_try_NUL_trans(yy_current_state);

                yy_bp = (yytext_ptr) + YY_MORE_ADJ;

                if (yy_next_state) {
                    /* Consume the NUL. */
                    yy_cp = ++(yy_c_buf_p);
                    yy_current_state = yy_next_state;
                    goto yy_match;
                }

                else {
                    yy_cp = (yy_c_buf_p);
                    goto yy_find_action;
                }
            }

            else {
                switch (yy_get_next_buffer()) {
                case EOB_ACT_END_OF_FILE: {
                    (yy_did_buffer_switch_on_eof) = 0;

                    if (sswrap()) {
                        /* Note: because we've taken care in
                                         * yy_get_next_buffer() to have set up
                                         * sstext, we can now set up
                                         * yy_c_buf_p so that if some total
                                         * hoser (like flex itself) wants to
                                         * call the scanner after we return the
                                         * YY_NULL, it'll still work - another
                                         * YY_NULL will get returned.
                                         */
                        (yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

                        yy_act = YY_STATE_EOF(YY_START);
                        goto do_action;
                    }

                    else {
                        if (!(yy_did_buffer_switch_on_eof)) {
                            YY_NEW_FILE;
                        }
                    }
                    break;
                }

                case EOB_ACT_CONTINUE_SCAN:
                    (yy_c_buf_p) =
                        (yytext_ptr) + yy_amount_of_matched_text;

                    yy_current_state = yy_get_previous_state();

                    yy_cp = (yy_c_buf_p);
                    yy_bp = (yytext_ptr) + YY_MORE_ADJ;
                    goto yy_match;

                case EOB_ACT_LAST_MATCH:
                    (yy_c_buf_p) =
                        &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

                    yy_current_state = yy_get_previous_state();

                    yy_cp = (yy_c_buf_p);
                    yy_bp = (yytext_ptr) + YY_MORE_ADJ;
                    goto yy_find_action;
                }
            }
            break;
        }

        default:
            YY_FATAL_ERROR(
                "fatal flex scanner internal error--no action found");
        } /* end of action switch */
    } /* end of scanning one token */
} /* end of sslex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *      EOB_ACT_LAST_MATCH -
 *      EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *      EOB_ACT_END_OF_FILE - end of file
 */
#if defined(__GNUC__) && !defined(__clang__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wanalyzer-malloc-leak"
#endif
static int yy_get_next_buffer(void)
{
    register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
    register char *source = (yytext_ptr);
    register int number_to_move, i;
    int ret_val;

    if ((yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1]) {
        YY_FATAL_ERROR(
            "fatal flex scanner internal error--end of buffer missed");
    }

    if (YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0) { /* Don't try to fill the buffer, so this is an EOF. */
        if ((yy_c_buf_p) - (yytext_ptr)-YY_MORE_ADJ == 1) {
            /* We matched a single character, the EOB, so
                         * treat this as a final EOF.
                         */
            return EOB_ACT_END_OF_FILE;
        }

        else {
            /* We matched some text prior to the EOB, first
                         * process it.
                         */
            return EOB_ACT_LAST_MATCH;
        }
    }

    /* Try to read more data. */

    /* First move last chars to start of buffer. */
    number_to_move = (int)((yy_c_buf_p) - (yytext_ptr)) - 1;

    for (i = 0; i < number_to_move; ++i) {
        *(dest++) = *(source++);
    }

    if (YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING) {
        /* don't do the read, it's not guaranteed to return an EOF,
                 * just force an EOF
                 */
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;
    }

    else {
        size_t num_to_read =
            YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

        while (num_to_read <= 0) { /* Not enough room in the buffer - grow it. */

            /* just a shorter name for the current buffer */
            YY_BUFFER_STATE b = YY_CURRENT_BUFFER;
            if (!b) {
                YY_FATAL_ERROR(
                    "fatal error - something is badly broken");
            }

            int yy_c_buf_p_offset =
                (int)((yy_c_buf_p)-b->yy_ch_buf);

            if (b->yy_is_our_buffer) {
                size_t new_size = b->yy_buf_size * 2;

                if (new_size <= 0) {
                    b->yy_buf_size += b->yy_buf_size / 8;
                } else {
                    b->yy_buf_size *= 2;
                }

                b->yy_ch_buf = (char *)
                    /* Include room in for 2 EOB chars. */
                    ssrealloc((void *)b->yy_ch_buf, b->yy_buf_size + 2);
            } else {
                /* Can't grow it, we don't own it. */
                b->yy_ch_buf = 0;
            }

            if (!b->yy_ch_buf) {
                YY_FATAL_ERROR(
                    "fatal error - scanner input buffer overflow");
            }

            (yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

            num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
                          number_to_move - 1;
        }

        if (num_to_read > YY_READ_BUF_SIZE) {
            num_to_read = YY_READ_BUF_SIZE;
        }

        /* Read in more data. */
        YY_INPUT((&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
                 (yy_n_chars), num_to_read);

        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
    }

    if ((yy_n_chars) == 0) {
        if (number_to_move == YY_MORE_ADJ) {
            ret_val = EOB_ACT_END_OF_FILE;
            ssrestart(ssin);
        }

        else {
            ret_val = EOB_ACT_LAST_MATCH;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
                YY_BUFFER_EOF_PENDING;
        }
    }

    else {
        ret_val = EOB_ACT_CONTINUE_SCAN;
    }

    if ((yy_size_t)((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
        /* Extend the array by 50%, plus the number we really need. */
        yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
        YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *)ssrealloc((void *)YY_CURRENT_BUFFER_LVALUE->yy_ch_buf, new_size);
        if (!YY_CURRENT_BUFFER_LVALUE->yy_ch_buf) {
            YY_FATAL_ERROR("out of dynamic memory in yy_get_next_buffer()");
        }
    }

    (yy_n_chars) += number_to_move;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

    (yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

    return ret_val;
}
#if defined(__GNUC__) && !defined(__clang__)
#pragma GCC diagnostic pop
#endif

/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state(void)
{
    register yy_state_type yy_current_state;
    register char *yy_cp;

    yy_current_state = (yy_start);

    for (yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp) {
        register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
        if (yy_accept[yy_current_state]) {
            (yy_last_accepting_state) = yy_current_state;
            (yy_last_accepting_cpos) = yy_cp;
        }
        while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
            yy_current_state = (int)yy_def[yy_current_state];
            if (yy_current_state >= 56) {
                yy_c = yy_meta[(unsigned int)yy_c];
            }
        }
        yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int)yy_c];
    }

    return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *      next_state = yy_try_NUL_trans( current_state );
 */
static yy_state_type yy_try_NUL_trans(yy_state_type yy_current_state)
{
    register int yy_is_jam;
    register char *yy_cp = (yy_c_buf_p);

    register YY_CHAR yy_c = 1;
    if (yy_accept[yy_current_state]) {
        (yy_last_accepting_state) = yy_current_state;
        (yy_last_accepting_cpos) = yy_cp;
    }
    while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
        yy_current_state = (int)yy_def[yy_current_state];
        if (yy_current_state >= 56) {
            yy_c = yy_meta[(unsigned int)yy_c];
        }
    }
    yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int)yy_c];
    yy_is_jam = (yy_current_state == 55);

    return yy_is_jam ? 0 : yy_current_state;
}

static void yyunput(int c, register char *yy_bp)
{
    register char *yy_cp;

    yy_cp = (yy_c_buf_p);

    /* undo effects of setting up sstext */
    *yy_cp = (yy_hold_char);

    if (yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2) { /* need to shift things up to make room */
        /* +2 for EOB chars. */
        register int number_to_move = (yy_n_chars) + 2;
        register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
        register char *source =
            &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

        while (source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf) {
            *--dest = *--source;
        }

        yy_cp += (int)(dest - source);
        yy_bp += (int)(dest - source);
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
            (yy_n_chars) = (int)YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

        if (yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2) {
            YY_FATAL_ERROR("flex scanner push-back overflow");
        }
    }

    *--yy_cp = (char)c;

    (yytext_ptr) = yy_bp;
    (yy_hold_char) = *yy_cp;
    (yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput(void)
#else
static int input(void)
#endif

{
    int c;

    *(yy_c_buf_p) = (yy_hold_char);

    if (*(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR) {
        /* yy_c_buf_p now points to the character we want to return.
                 * If this occurs *before* the EOB characters, then it's a
                 * valid NUL; if not, then we've hit the end of the buffer.
                 */
        if ((yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)]) {
            /* This was really a NUL. */
            *(yy_c_buf_p) = '\0';
        }

        else { /* need more input */
            ptrdiff_t offset = (yy_c_buf_p) - (yytext_ptr);
            ++(yy_c_buf_p);

            switch (yy_get_next_buffer()) {
            case EOB_ACT_LAST_MATCH:
                /* This happens because yy_g_n_b()
                                         * sees that we've accumulated a
                                         * token and flags that we need to
                                         * try matching the token before
                                         * proceeding.  But for input(),
                                         * there's no matching to consider.
                                         * So convert the EOB_ACT_LAST_MATCH
                                         * to EOB_ACT_END_OF_FILE.
                                         */

                /* Reset buffer status. */
                ssrestart(ssin);

                /*FALLTHROUGH*/

            case EOB_ACT_END_OF_FILE: {
                if (sswrap()) {
                    return EOF;
                }

                if (!(yy_did_buffer_switch_on_eof)) {
                    YY_NEW_FILE;
                }
#ifdef __cplusplus
                return yyinput();
#else
                return input();
#endif
            }

            case EOB_ACT_CONTINUE_SCAN:
                (yy_c_buf_p) = (yytext_ptr) + offset;
                break;
            }
        }
    }

    c = *(unsigned char *)(yy_c_buf_p); /* cast for 8-bit char's */
    *(yy_c_buf_p) = '\0';               /* preserve sstext */
    (yy_hold_char) = *++(yy_c_buf_p);

    return c;
}
#endif /* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 *
 * @note This function does not reset the start condition to @c INITIAL .
 */
void ssrestart(FILE *input_file)
{
    if (!YY_CURRENT_BUFFER) {
        ssensure_buffer_stack();
        YY_CURRENT_BUFFER_LVALUE =
            ss_create_buffer(ssin, YY_BUF_SIZE);
    }

    ss_init_buffer(YY_CURRENT_BUFFER, input_file);
    ss_load_buffer_state();
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 *
 */
void ss_switch_to_buffer(YY_BUFFER_STATE new_buffer)
{
    /* TODO. We should be able to replace this entire function body
         * with
         *              sspop_buffer_state();
         *              sspush_buffer_state(new_buffer);
     */
    ssensure_buffer_stack();
    if (YY_CURRENT_BUFFER == new_buffer) {
        return;
    }

    if (YY_CURRENT_BUFFER) {
        /* Flush out information for old buffer. */
        *(yy_c_buf_p) = (yy_hold_char);
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
    }

    /* coverity[var_deref_op] */
    YY_CURRENT_BUFFER_LVALUE = new_buffer;
    ss_load_buffer_state();

    /* We don't actually know whether we did this switch during
         * EOF (sswrap()) processing, but the only time this flag
         * is looked at is after sswrap() is called, so it's safe
         * to go ahead and always set it.
         */
    (yy_did_buffer_switch_on_eof) = 1;
}

static void ss_load_buffer_state(void)
{
    (yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
    (yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
    ssin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
    (yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 *
 * @return the allocated buffer state.
 */
YY_BUFFER_STATE ss_create_buffer(FILE *file, int size)
{
    YY_BUFFER_STATE b;

    b = (YY_BUFFER_STATE)ssalloc(sizeof(struct yy_buffer_state));
    if (!b) {
        YY_FATAL_ERROR("out of dynamic memory in ss_create_buffer()");
    }

    b->yy_buf_size = size;

    /* yy_ch_buf has to be 2 characters longer than the size given because
         * we need to put in 2 end-of-buffer characters.
         */
    b->yy_ch_buf = (char *)ssalloc(b->yy_buf_size + 2);
    if (!b->yy_ch_buf) {
        free(b);
        YY_FATAL_ERROR("out of dynamic memory in ss_create_buffer()");
    }

    b->yy_is_our_buffer = 1;

    ss_init_buffer(b, file);

    return b;
}

/** Destroy the buffer.
 * @param b a buffer created with ss_create_buffer()
 *
 */
void ss_delete_buffer(YY_BUFFER_STATE b)
{
    if (!b) {
        return;
    }

    if (b == YY_CURRENT_BUFFER) { /* Not sure if we should pop here. */
        YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE)0;
    }

    if (b->yy_is_our_buffer) {
        ssfree((void *)b->yy_ch_buf);
    }

    ssfree((void *)b);
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a ssrestart() or at EOF.
 */
static void ss_init_buffer(YY_BUFFER_STATE b, FILE *file)

{
    int oerrno = errno;

    if (!b) {
        return;
    }

    ss_flush_buffer(b);

    b->yy_input_file = file;
    b->yy_fill_buffer = 1;

    /* If b is the current buffer, then ss_init_buffer was _probably_
     * called from ssrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER) {
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

    b->yy_is_interactive = file ? (isatty(fileno(file)) > 0) : 0;

    errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 *
 */
void ss_flush_buffer(YY_BUFFER_STATE b)
{
    if (!b) {
        return;
    }

    b->yy_n_chars = 0;

    /* We always need two end-of-buffer characters.  The first causes
         * a transition to the end-of-buffer state.  The second causes
         * a jam in that state.
         */
    b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
    b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

    b->yy_buf_pos = &b->yy_ch_buf[0];

    b->yy_at_bol = 1;
    b->yy_buffer_status = YY_BUFFER_NEW;

    if (b == YY_CURRENT_BUFFER) {
        ss_load_buffer_state();
    }
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *
 */
void sspush_buffer_state(YY_BUFFER_STATE new_buffer)
{
    if (new_buffer == NULL) {
        return;
    }

    ssensure_buffer_stack();

    /* This block is copied from ss_switch_to_buffer. */
    if (YY_CURRENT_BUFFER) {
        /* Flush out information for old buffer. */
        *(yy_c_buf_p) = (yy_hold_char);
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
    }

    /* Only push if top exists. Otherwise, replace top. */
    if (YY_CURRENT_BUFFER) {
        (yy_buffer_stack_top)++;
    }
    /* coverity[var_deref_op] */
    YY_CURRENT_BUFFER_LVALUE = new_buffer;

    /* copied from ss_switch_to_buffer. */
    ss_load_buffer_state();
    (yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *
 */
void sspop_buffer_state(void)
{
    if (!YY_CURRENT_BUFFER) {
        return;
    }

    ss_delete_buffer(YY_CURRENT_BUFFER);
    YY_CURRENT_BUFFER_LVALUE = NULL;
    if ((yy_buffer_stack_top) > 0) {
        --(yy_buffer_stack_top);
    }

    if (YY_CURRENT_BUFFER) {
        ss_load_buffer_state();
        (yy_did_buffer_switch_on_eof) = 1;
    }
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void ssensure_buffer_stack(void)
{
    size_t num_to_alloc;

    if (!(yy_buffer_stack)) {
        /* First allocation is just for 2 elements, since we don't know if this
                 * scanner will even need a stack. We use 2 instead of 1 to avoid an
                 * immediate realloc on the next call.
         */
        num_to_alloc = 1;
        (yy_buffer_stack) = (struct yy_buffer_state **)ssalloc(num_to_alloc * sizeof(struct yy_buffer_state *));
        if (!(yy_buffer_stack)) {
            YY_FATAL_ERROR("out of dynamic memory in ssensure_buffer_stack()");
        }

        memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state *));

        (yy_buffer_stack_max) = num_to_alloc;
        (yy_buffer_stack_top) = 0;
        return;
    }

    if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1) {
        /* Increase the buffer to prepare for a possible push. */
        int grow_size = 8 /* arbitrary grow size */;

        num_to_alloc = (yy_buffer_stack_max) + grow_size;
        (yy_buffer_stack) = (struct yy_buffer_state **)ssrealloc((yy_buffer_stack),
                                                                 num_to_alloc * sizeof(struct yy_buffer_state *));
        if (!(yy_buffer_stack)) {
            YY_FATAL_ERROR("out of dynamic memory in ssensure_buffer_stack()");
        }

        /* zero only the new slots.*/
        memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state *));
        (yy_buffer_stack_max) = num_to_alloc;
    }
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 *
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE ss_scan_buffer(char *base, yy_size_t size)
{
    YY_BUFFER_STATE b;

    if (size < 2 ||
        base[size - 2] != YY_END_OF_BUFFER_CHAR ||
        base[size - 1] != YY_END_OF_BUFFER_CHAR) {
        /* They forgot to leave room for the EOB's. */
        return 0;
    }

    b = (YY_BUFFER_STATE)ssalloc(sizeof(struct yy_buffer_state));
    if (!b) {
        YY_FATAL_ERROR("out of dynamic memory in ss_scan_buffer()");
    }

    b->yy_buf_size = size - 2; /* "- 2" to take care of EOB's */
    b->yy_buf_pos = b->yy_ch_buf = base;
    b->yy_is_our_buffer = 0;
    b->yy_input_file = 0;
    b->yy_n_chars = (int)b->yy_buf_size;
    b->yy_is_interactive = 0;
    b->yy_at_bol = 1;
    b->yy_fill_buffer = 0;
    b->yy_buffer_status = YY_BUFFER_NEW;

    ss_switch_to_buffer(b);

    return b;
}

/** Setup the input buffer state to scan a string. The next call to sslex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 *
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       ss_scan_bytes() instead.
 */
YY_BUFFER_STATE ss_scan_string(yyconst char *yystr)
{
    return ss_scan_bytes(yystr, (int)strlen(yystr));
}

/** Setup the input buffer state to scan the given bytes. The next call to sslex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 *
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE ss_scan_bytes(yyconst char *yybytes, int _yybytes_len)
{
    YY_BUFFER_STATE b;
    char *buf;
    yy_size_t n;
    int i;

    /* Get memory for full buffer, including space for trailing EOB's. */
    n = _yybytes_len + 2;
    buf = (char *)ssalloc(n);
    if (!buf) {
        YY_FATAL_ERROR("out of dynamic memory in ss_scan_bytes()");
    }

    for (i = 0; i < _yybytes_len; ++i) {
        buf[i] = yybytes[i];
    }

    buf[_yybytes_len] = buf[_yybytes_len + 1] = YY_END_OF_BUFFER_CHAR;

    b = ss_scan_buffer(buf, n);
    if (!b) {
        free(buf);
        YY_FATAL_ERROR("bad buffer in ss_scan_bytes()");
    }

    /* It's okay to grow etc. this buffer, and we should throw it
         * away when we're done.
         */
    b->yy_is_our_buffer = 1;

    return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error(yyconst char *msg)
{
    (void)fprintf(stderr, "%s\n", msg);
    exit(YY_EXIT_FAILURE);
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n)                                 \
    do {                                          \
        /* Undo effects of setting up sstext. */  \
        int yyless_macro_arg = (n);               \
        YY_LESS_LINENO(yyless_macro_arg);         \
        sstext[ssleng] = (yy_hold_char);          \
        (yy_c_buf_p) = sstext + yyless_macro_arg; \
        (yy_hold_char) = *(yy_c_buf_p);           \
        *(yy_c_buf_p) = '\0';                     \
        ssleng = yyless_macro_arg;                \
    } while (0)

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 *
 */
int ssget_lineno(void)
{
    return sslineno;
}

/** Get the input stream.
 *
 */
FILE *ssget_in(void)
{
    return ssin;
}

/** Get the output stream.
 *
 */
FILE *ssget_out(void)
{
    return ssout;
}

/** Get the length of the current token.
 *
 */
int ssget_leng(void)
{
    return ssleng;
}

/** Get the current token.
 *
 */

char *ssget_text(void)
{
    return sstext;
}

/** Set the current line number.
 * @param line_number
 *
 */
void ssset_lineno(int line_number)
{
    sslineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 *
 * @see ss_switch_to_buffer
 */
void ssset_in(FILE *in_str)
{
    ssin = in_str;
}

void ssset_out(FILE *out_str)
{
    ssout = out_str;
}

int ssget_debug(void)
{
    return ss_flex_debug;
}

void ssset_debug(int bdebug)
{
    ss_flex_debug = bdebug;
}

static int yy_init_globals(void)
{
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from sslex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *)0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    ssin = stdin;
    ssout = stdout;
#else
    ssin = (FILE *)0;
    ssout = (FILE *)0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * sslex_init()
     */
    return 0;
}

/* sslex_destroy is for both reentrant and non-reentrant scanners. */
int sslex_destroy(void)
{
    /* Pop the buffer stack, destroying each element. */
    while (YY_CURRENT_BUFFER) {
        ss_delete_buffer(YY_CURRENT_BUFFER);
        YY_CURRENT_BUFFER_LVALUE = NULL;
        sspop_buffer_state();
    }

    /* Destroy the stack itself. */
    ssfree((yy_buffer_stack));
    (yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * sslex() is called, initialization will occur. */
    yy_init_globals();

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy(char *s1, yyconst char *s2, int n)
{
    register int i;
    for (i = 0; i < n; ++i) {
        s1[i] = s2[i];
    }
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen(yyconst char *s)
{
    register int n;
    for (n = 0; s[n]; ++n)
        ;

    return n;
}
#endif

void *ssalloc(yy_size_t size)
{
    return (void *)malloc(size);
}

void *ssrealloc(void *ptr, yy_size_t size)
{
    /* The cast to (char *) in the following accommodates both
         * implementations that use char* generic pointers, and those
         * that use void* generic pointers.  It works with the latter
         * because both ANSI C and C++ allow castless assignment from
         * any pointer type to void*, and deal with argument conversions
         * as though doing an assignment.
         */
    return (void *)realloc((char *)ptr, size);
}

void ssfree(void *ptr)
{
    free((char *)ptr); /* see ssrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

int sswrap(void)
{
    return 1;
}
#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic pop
#endif

// NOLINTEND
