/* Generated file (by generator) */

/*
 * SPDX-FileCopyrightText: 2015 William Yu <williamyu@gnome.org>
 *
 * SPDX-License-Identifier: LGPL-2.1-only OR MPL-2.0
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "i-cal-limits.h"
#include "libical-glib-private.h"

/**
 * i_cal_limit_set:
 * @kind: The kind of limit to set
 * @limit: The limit value. Use SIZE_MAX (i.e. maximum value of size_t) to set an unlimited value.
 *
 * Sets the value for the corresponding internal library limit.
 *
 * Since: 4.0
 *
 **/
void
i_cal_limit_set (ICalLimitsKind kind, 
                 size_t limit)
{
	icallimit_set ((icallimits_kind) (kind), limit);
}

/**
 * i_cal_limit_get:
 * @kind: The kind of limit to set
 *
 * Gets the value for the corresponding internal library limit.
 *
 * Returns: The current value of the specified #ICalLimitsKind
 *
 * Since: 4.0
 *
 **/
size_t
i_cal_limit_get (ICalLimitsKind kind)
{
	return icallimit_get ((icallimits_kind) (kind));
}
